<?php

namespace WPMC\Controller;

use GuzzleHttp\Psr7\ServerRequest;
use WPMC\Plugin;
use WPMC\Service\MyChannelsApiService;

class AdminController
{
    const SETTINGS_PAGE_SLUG = 'wp-mychannels';
    const SETTINGS_SECTION = 'default';

    public static function setup()
    {

        //Load translations
        add_action('plugins_loaded', array(self::class, 'loadTranslations'));

        //Add Settings link to plugin page
        add_filter('plugin_action_links_' . WPMC_PLUGIN_FILE, [self::class, 'addActionLinks'], 10, 1);

        //Add Settings page in Admin Menu
        add_action('admin_menu', [self::class, 'registerSettingsPage']);

        //Register settings
        add_action('admin_init', [self::class, 'registerSettingsFields']);

        //Add listener before settings change save
        add_filter('pre_update_option_' . Plugin::SETTINGS_OPTION_KEY, [self::class, 'beforeSettingsUpdate'], 10, 2);

        //Add listener after settings change
        add_action('update_option_' . Plugin::SETTINGS_OPTION_KEY, [self::class, 'afterSettingsUpdate'], 10, 2);

    }

    public static function loadTranslations(){
        load_plugin_textdomain(Plugin::TRANSLATION_DOMAIN, false, WPMC_PLUGIN_DIRNAME . '/languages/');
    }


    public static function registerSettingsPage()
    {
        #add_menu_page('WP MyChannels', 'WP MyChannels', Plugin::REQUIRED_ADMIN_ROLE, self::SETTINGS_PAGE_SLUG, [self::class, 'renderSettingsPage'], 'dashicons-format-video');
        add_options_page('WP MyChannels', 'WP MyChannels', Plugin::REQUIRED_ADMIN_CAPABILITY, self::SETTINGS_PAGE_SLUG, [self::class, 'renderSettingsPage']);
    }


    public static function renderSettingsPage()
    {

        echo '<div class="wrap">';
            echo '<h1>' . esc_html(get_admin_page_title()) . '</h1>';
            //echo '<h2>Instellingen</h2>';

            echo '<div id="poststuff" style="margin-top: 0">';
                echo '<div id="post-body" class="metabox-holder columns-2">';

                    echo '<div id="post-body-content" style="">';
                        echo '<div class="stuffbox" style="padding: 20px">';
                            echo '<div style="font-size: 20px">'.__('Settings').'</div>';
                            /** @noinspection HtmlUnknownTarget */
                            echo '<form action="options.php" method="post">';

                                settings_fields(self::SETTINGS_PAGE_SLUG);
                                do_settings_sections(self::SETTINGS_PAGE_SLUG);

                                echo '<div style="margin-top: 1rem">';
                                    submit_button(__('Save'), 'primary', 'submit', false);
                                echo '</div>';
                            echo '</form>';
                        echo '</div>';
                    echo '</div>';

                    echo '<div id="postbox-container-1" class="postbox-container">';

                        echo '<div class="stuffbox" style="padding: 16px 12px">';

                            echo '<div style="font-size: 20px; margin-bottom: 1rem;">'.__('Status').'</div>';

                            $configValid = Plugin::configIsValid();
                            $isHttps = Plugin::connectionIsHttps();

                            if($configValid && $isHttps) {

                                echo '<div style="font-size: 16px; margin-bottom: 1rem;"><span>✅</span>&nbsp;'.__('Settings') .' OK</div>';

                                $frontendUrl = get_home_url().'/'.Plugin::getSetting(Plugin::SETTING_MOUNTPOINT);
                                echo '<a href="'.$frontendUrl.'" target="_blank" class="button" style="margin-bottom: .5rem;">' . __('View page') . '</a>';

                                echo '&nbsp;&nbsp;';

                                $sanityCheckUrl = $frontendUrl.'/sanity';
                                echo '<a href="'.$sanityCheckUrl.'" target="_blank" class="button" style="margin-bottom: .5rem;">' . __('Sanity check') . '</a>';

                            }
                            else {
                                echo '<div style="font-size: 16px; margin-bottom: 1rem;"><span>❌</span> '.__('Configuration Error').'</div>';

                                echo '<ul>';
                                    if (!$configValid) echo '<li>&bull; '.__('Settings are invalid.', 'wp-mychannels').'</li>';
                                    if (!$isHttps) echo '<li>&bull; '.__('Connection must be HTTPS.', 'wp-mychannels').'</li>';
                                echo '</ul>';
                            }


                            if(Plugin::getSetting(Plugin::SETTING_LOGGING)) {
                                $logFileUri = Plugin::getLogFileUrl();
                                echo '<a href="'.$logFileUri.'" target="_blank" class="button" style="margin-bottom: .5rem;">'.
                                     __('Download log file', 'wp-mychannels')
                                    .'</a>';
                            }


                        echo '</div>';
                    echo '</div>';
                echo '</div>';

            echo '</div>';
        echo '</div>';
    }

    public static function registerSettingsFields()
    {


        register_setting(self::SETTINGS_PAGE_SLUG, Plugin::SETTINGS_OPTION_KEY, [
            'sanitize_callback' => [self::class, 'onSettingsFormSubmit'],
        ]);

        add_settings_section(
            self::SETTINGS_SECTION,
            '',
            null,
            self::SETTINGS_PAGE_SLUG
        );

        add_settings_field(
            Plugin::SETTING_HOSTNAME,
            __('Hostname'),
            [self::class, 'renderTextFieldHtml'],
            self::SETTINGS_PAGE_SLUG,
            self::SETTINGS_SECTION,
            [
                'label_for' => Plugin::SETTING_HOSTNAME,
                'field_classes' => ['code'],
            ]
        );

        add_settings_field(
            Plugin::SETTING_MOUNTPOINT,
            __('Mountpoint', 'wp-mychannels'),
            [self::class, 'renderTextFieldHtml'],
            self::SETTINGS_PAGE_SLUG,
            self::SETTINGS_SECTION,
            [
                'label_for' => Plugin::SETTING_MOUNTPOINT,
                'field_classes' => ['code'],
            ]
        );

        add_settings_field(
            Plugin::SETTING_USERNAME,
            __('Username', 'wp-mychannels'),
            [self::class, 'renderTextFieldHtml'],
            self::SETTINGS_PAGE_SLUG,
            'default',
            [
                'label_for' => Plugin::SETTING_USERNAME,
                'field_classes' => ['code'],
            ]
        );

        add_settings_field(
            Plugin::SETTING_PASSWORD,
            __('Password', 'wp-mychannels'),
            [self::class, 'renderTextFieldHtml'],
            self::SETTINGS_PAGE_SLUG,
            self::SETTINGS_SECTION,
            [
                'label_for' => Plugin::SETTING_PASSWORD,
                'field_classes' => ['code'],
            ]
        );

        add_settings_field(
            Plugin::SETTING_LOGGING,
            __('Enable logging', 'wp-mychannels'),
            [self::class, 'renderCheckboxHtml'],
            self::SETTINGS_PAGE_SLUG,
            self::SETTINGS_SECTION,
            [
                'label_for' => Plugin::SETTING_LOGGING,
                'type' => 'checkbox',
            ]
        );


    }


    public static function onSettingsFormSubmit($values)
    {
        //Checkbox values
        global $wp_settings_fields;
        $wpmcSettingFields = $wp_settings_fields[self::SETTINGS_PAGE_SLUG][self::SETTINGS_SECTION];

        $checkboxFields = array_keys(array_filter($wpmcSettingFields, function($field){
            return (isset($field['args']) && isset($field['args']['type']) && $field['args']['type'] === 'checkbox');
        }));

        foreach($checkboxFields as $fieldName){
            $values[$fieldName] = (array_key_exists($fieldName, $values));
        }

        if((filter_var($values[Plugin::SETTING_HOSTNAME], FILTER_VALIDATE_URL) !== false)) {
            //Strip https
            $values[Plugin::SETTING_HOSTNAME] = parse_url($values[Plugin::SETTING_HOSTNAME], PHP_URL_HOST);
        }

        if($values[Plugin::SETTING_MOUNTPOINT] === '' || $values[Plugin::SETTING_HOSTNAME] === '') {
            $isValid = false;
        }
        else {
            $isValid = self::validateSettings($values);
        }

        if ($isValid) {
            $msg = __('Settings saved.');
            add_settings_error(self::SETTINGS_PAGE_SLUG, 'wpmc_message', $msg, 'updated');
        }
        else {
            $errorMsg = __('Settings are invalid.', 'wp-mychannels'). ' '.__('Please try again.');
            add_settings_error(self::SETTINGS_PAGE_SLUG, 'wpmc_message', $errorMsg, 'error');
        }

        $values[Plugin::SETTING_CONFIG_VALID] = $isValid;

        return $values;

    }

    public static function validateSettings($settings){

        try {

            $logFile = ($settings[Plugin::SETTING_LOGGING]) ? Plugin::getLogFilePath() : null;

            $api = new MyChannelsApiService($settings[Plugin::SETTING_HOSTNAME],
                                            $settings[Plugin::SETTING_USERNAME],
                                            $settings[Plugin::SETTING_PASSWORD],
                                            $settings[Plugin::SETTING_SITE_IDENTIFIER],
                                            $logFile);


            $serverRequest = ServerRequest::fromGlobals();
            $apiResponse = $api->testSettings($settings[Plugin::SETTING_MOUNTPOINT], $serverRequest);

            $statusCode = $apiResponse->getStatusCode();

            if ($statusCode !== 200) throw new \RuntimeException('API Response code not 200');

            $responseBody = $apiResponse->getBody()->getContents();
            $responseData = json_decode($responseBody, true);

            if (json_last_error() !== JSON_ERROR_NONE) throw new \RuntimeException('API Response body is not valid JSON');

            if (!array_key_exists('html', $responseData)) throw new \RuntimeException('html not present in JSON response');

            return true;

        }
        catch(\RuntimeException $e) {
            if($settings[Plugin::SETTING_LOGGING]) {
                error_log((string)$e.PHP_EOL, 3, Plugin::getLogFilePath());
            }
            return false;
        }

    }

    public static function beforeSettingsUpdate($newValues, $oldValues)
    {
        //Merge existing old values (e.g. site_identifier)
        if(is_array($oldValues) && is_array($newValues)) {
            $newValues = array_merge($oldValues, $newValues);
        }

        return $newValues;

    }


    public static function afterSettingsUpdate($oldValues, $newValues)
    {
        if ($oldValues[Plugin::SETTING_MOUNTPOINT] !== $newValues[Plugin::SETTING_MOUNTPOINT]) {
            self::updateRewriteRules($oldValues[Plugin::SETTING_MOUNTPOINT], $newValues[Plugin::SETTING_MOUNTPOINT]);
        }
        if(! $newValues[Plugin::SETTING_LOGGING]) {
            $logFile = Plugin::getLogFilePath();
            if(file_exists($logFile)){
                unlink($logFile);
            }
        }
    }



    public static function updateRewriteRules($oldMountpoint, $newMountpoint){

        if($oldMountpoint) {
            //Remove the old rewrite Rule

            /** @var \WP_Rewrite $wp_rewrite */
            global $wp_rewrite;
            if(array_key_exists(Plugin::getRewriteRuleRegex($oldMountpoint), $wp_rewrite->extra_rules_top)) {
                unset($wp_rewrite->extra_rules_top[Plugin::getRewriteRuleRegex($oldMountpoint)]);
            }

        }

        if(Plugin::configIsValid()) {

            //Add the new rewrite rule
            add_rewrite_rule(
                Plugin::getRewriteRuleRegex($newMountpoint),
                Plugin::FRONTEND_REWRITE_QUERY,
                'top'
            );

        }

        flush_rewrite_rules(false);

    }



    public static function addActionLinks($links)
    {

        $extra_links = [
            '<a href="' . admin_url('options-general.php?page='.self::SETTINGS_PAGE_SLUG ) . '">'.__('Settings').'</a>'
        ];

        return array_merge($extra_links, $links);

    }


    public static function renderTextFieldHtml($args)
    {
        $value = Plugin::getSetting($args['label_for']);
        if (null === $value && isset($args['default_value'])) $value = $args['default_value'];
        $classStr = is_array($args['field_classes']) ? implode(' ',$args['field_classes']) : '';

        echo '<input id="'.$args['label_for'].'" class="regular-text '.$classStr.'" value="' . ($value) . '" name="'.Plugin::SETTINGS_OPTION_KEY.'[' . $args['label_for'] . ']" />';
        if (isset($args['description']) && $args['description']) echo '<p class="description">' . $args['description'] . '</p>';

    }

    public static function renderCheckboxHtml($args)
    {
        $checked = Plugin::getSetting($args['label_for']);
        if (null === $checked && isset($args['default_value'])) $checked = $args['default_value'];

        $classStr = is_array($args['field_classes']) ? implode(' ',$args['field_classes']) : '';
        $checkedStr = $checked ? 'checked="checked"':'';

        echo '<input type="checkbox" id="'.$args['label_for'].'" class="'.$classStr.'" 
                value="1" name="'.Plugin::SETTINGS_OPTION_KEY.'[' . $args['label_for'] . ']"' .$checkedStr.' />';

    }


}
