<?php

namespace WPMC;

use WPMC\Controller\AdminController;
use WPMC\Controller\FrontendController;

class Plugin
{

    private static $settings;

    const FRONTEND_QUERY_VAR = 'wpmc';
    const FRONTEND_REWRITE_QUERY = 'index.php?'.self::FRONTEND_QUERY_VAR.'=$matches[1]';

    const SETTINGS_OPTION_KEY = 'wpmc_settings';

    const SETTING_MOUNTPOINT = 'mountpoint';
    const SETTING_HOSTNAME = 'hostname';
    const SETTING_USERNAME = 'username';
    const SETTING_PASSWORD = 'password';
    const SETTING_LOGGING = 'logging';
    const SETTING_SITE_IDENTIFIER = 'site_identifier';
    const SETTING_CONFIG_VALID = 'config_valid';

    const REQUIRED_ADMIN_CAPABILITY = 'manage_options';

    const TAG_ACTION_FRONTEND_OUTPUT_BODY = 'wpmc_body';
    const TAG_FILTER_FRONTEND_TEMPLATE_PATH = 'wpmc_template';
    const TAG_FILTER_FRONTEND_BEFORE_BODY = 'wpmc_before_body';
    const TAG_FILTER_FRONTEND_AFTER_BODY = 'wpmc_after_body';

    const TRANSLATION_DOMAIN = 'wp-mychannels';

    const LOG_FILE = 'wp_mychannels.log';

    public static function setup()
    {
        //Set default settings on plugin activation
        register_activation_hook(WPMC_PLUGIN_FILE, [self::class, 'setDefaultSettings']);

        //Load settings on init
        self::loadSettings();

        //Reload settings after update
        add_action('update_option_' . Plugin::SETTINGS_OPTION_KEY, [self::class, 'loadSettings'], 1, 0);

        AdminController::setup();

        if(Plugin::configIsValid()) {
            FrontendController::setup();
        }

    }


    public static function loadSettings(){
        self::$settings = get_option(self::SETTINGS_OPTION_KEY);
    }

    public static function getSetting($key, $default = null) {
        return (isset(self::$settings) && isset(self::$settings[$key])) ? self::$settings[$key] : $default;
    }

    public static function getSettings() {
        return self::$settings ?: [];
    }

    public static function configIsValid(){
        return self::getSetting(Plugin::SETTING_CONFIG_VALID);
    }

    public static function connectionIsHttps(){
        return isset($_SERVER['HTTPS']);
    }



    public static function setDefaultSettings() {

        $settings = self::getSettings();

        $changed = false;

        if(null === self::getSetting(self::SETTING_HOSTNAME)) {

            $siteHostname = (isset($_SERVER['HTTP_HOST'])) ? $_SERVER['HTTP_HOST'] : '';
            preg_match('/([^.]+)\.[^.]+$/', $siteHostname, $hostnameComponents);
            $defaultApiHostname = $hostnameComponents[1].'.mychannels.video';

            $settings[Plugin::SETTING_HOSTNAME] = $defaultApiHostname;
            $changed = true;

        }

        if(null === self::getSetting(self::SETTING_MOUNTPOINT)) {
            $settings[self::SETTING_MOUNTPOINT] = 'video';
            $changed = true;
        }


        if(null === self::getSetting(self::SETTING_SITE_IDENTIFIER)) {

            if(isset($_SERVER['HTTP_HOST'])) {
                $siteIdentifier = str_replace('.','_', strtolower($_SERVER['HTTP_HOST']));
            }
            else {
                $siteIdentifier = 'unknown';
            }

            $settings[self::SETTING_SITE_IDENTIFIER] = $siteIdentifier;
            $changed = true;
        }


        if(null === self::getSetting(self::SETTING_LOGGING)) {
            $settings[self::SETTING_LOGGING] = false;
            $changed = true;
        }

        if($changed) {
            $valid = AdminController::validateSettings($settings);
            $settings[Plugin::SETTING_CONFIG_VALID] = $valid;
            update_option(Plugin::SETTINGS_OPTION_KEY, $settings);
        }

    }

    public static function getRewriteRuleRegex($mountpoint){
        return '^'.$mountpoint.'[/]?(.*)$';
    }


    public static function getLogFileUrl(){
        return wp_get_upload_dir()['baseurl'].'/'.self::LOG_FILE;
    }

    public static function getLogFilePath(){
        return wp_get_upload_dir()['basedir'].DIRECTORY_SEPARATOR.self::LOG_FILE;
    }



}
